import json

import pandas as pd
from .base import BaseFileHandler

class XlsHandler(BaseFileHandler):
    str_like = False
    
    def load_from_fileobj(self, file, **kwargs):
        df = pd.read_excel(file,**kwargs)
        return df.to_dict('records')
    
    def dump_to_fileobj(self, obj, file, **kwargs): 
        df = pd.DataFrame(data = obj)
        df.to_excel(file,index=0,**kwargs)

    def dump_to_str(self, obj, **kwargs): 
        return json.dumps(obj, **kwargs)



# Copyright (c) OpenMMLab. All rights reserved.
import sys
import time
import warnings
from getpass import getuser
from socket import gethostname
from types import ModuleType
from typing import Optional 
import numpy as np
import random,os
import pipetool


def get_host_info() -> str:
    """Get hostname and username.

    Return empty string if exception raised, e.g. ``getpass.getuser()`` will
    lead to error in docker container
    """
    host = ''
    try:
        host = f'{getuser()}@{gethostname()}'
    except Exception as e:
        warnings.warn(f'Host or user not found: {str(e)}')
    finally:
        return host


def get_time_str() -> str:
    return time.strftime('%Y%m%d_%H%M%S', time.localtime())


def obj_from_dict(info: dict,
                  parent: Optional[ModuleType] = None,
                  default_args: Optional[dict] = None):
    """Initialize an object from dict.

    The dict must contain the key "type", which indicates the object type, it
    can be either a string or type, such as "list" or ``list``. Remaining
    fields are treated as the arguments for constructing the object.

    Args:
        info (dict): Object types and arguments.
        parent (:class:`module`): Module which may containing expected object
            classes.
        default_args (dict, optional): Default arguments for initializing the
            object.

    Returns:
        any type: Object built from the dict.
    """
    assert isinstance(info, dict) and 'type' in info
    assert isinstance(default_args, dict) or default_args is None
    args = info.copy()
    obj_type = args.pop('type')
    if pipetool.is_str(obj_type):
        if parent is not None:
            obj_type = getattr(parent, obj_type)
        else:
            obj_type = sys.modules[obj_type]
    elif not isinstance(obj_type, type):
        raise TypeError('type must be a str or valid type, but '
                        f'got {type(obj_type)}')
    if default_args is not None:
        for name, value in default_args.items():
            args.setdefault(name, value)
    return obj_type(**args)


def seed_everything(seed=None):
    max_seed_value = np.iinfo(np.uint32).max
    min_seed_value = np.iinfo(np.uint32).min

    if (seed is None) or not (min_seed_value <= seed <= max_seed_value):
        random.randint(np.iinfo(np.uint32).min, np.iinfo(np.uint32).max)
    print(f"Global seed set to {seed}")
    os.environ["PYTHONHASHSEED"] = str(seed)
    random.seed(seed)
    np.random.seed(seed)
    try:
        import torch 
        torch.manual_seed(seed)
        torch.cuda.manual_seed(seed)
        torch.cuda.manual_seed_all(seed)
    except:
        pass
    
    try:
        import paddle 
        paddle.seed(seed)
    except:
        pass
    
    try:
        import tensorflow as tf
        tf.random.set_seed(seed)
    except:
        pass
    
    return seed

def seg_generator(iterables, seg_len, seg_backoff=0):
    '''
    滑窗实现
    '''
    if seg_len <= 0:
        yield iterables, 0
    else:
        #  # 确保iterables列表中每一项的条目数相同
        #  assert sum([len(x)
        #              for x in iterables]) == len(iterables[0]) * len(iterables)
        assert iterables[0] is not None
        s0 = 0
        while s0 < len(iterables[0]):
            s1 = s0 + seg_len
            segs = [x[s0:s1] if x else None for x in iterables]
            yield segs, s0
            s0 += seg_len - seg_backoff


# flake8: noqa
# Copyright (c) OpenMMLab. All rights reserved.
from .config import Config, ConfigDict, DictAction
from .misc import (check_prerequisites, concat_list, deprecated_api_warning,
                   has_method, import_modules_from_strings, is_list_of,
                   is_method_overridden, is_seq_of, is_str, is_tuple_of,
                   iter_cast, list_cast, requires_executable, requires_package,
                   slice_list, to_1tuple, to_2tuple, to_3tuple, to_4tuple,
                   to_ntuple, tuple_cast)
from .path import (check_file_exist, fopen, is_filepath, mkdir_or_exist,
                   scandir, symlink)
from .progressbar import (ProgressBar, track_iter_progress,
                          track_parallel_progress, track_progress)
from .testing import (assert_attrs_equal, assert_dict_contains_subset,
                      assert_dict_has_keys, assert_is_norm_layer,
                      assert_keys_equal, assert_params_all_zeros,
                      check_python_script)
from .timer import Timer, TimerError, check_time
from .version_utils import digit_version, get_git_hash
from .registry import Registry, build_from_cfg
from .logging import get_logger, print_log
from .logger import logtime, log 
from .utils import get_host_info, get_time_str, obj_from_dict, seed_everything,seg_generator


# Copyright (c) OpenMMLab. All rights reserved.
from .base import BaseFileHandler
from .json_handler import JsonHandler
from .jsons_handler import JsonsHandler
from .pickle_handler import PickleHandler
from .yaml_handler import YamlHandler
from .csv_handler import CsvHandler
from .tsv_handler import TsvHandler
from .texts_handler import TextsHandler
from .xls_handler import XlsHandler

__all__ = ['BaseFileHandler', 'XlsHandler','JsonHandler','JsonsHandler', 'PickleHandler', 'YamlHandler','CsvHandler','TsvHandler','TextsHandler']



import json

import numpy as np

from .base import BaseFileHandler
from ..parse import list_from_file

def set_default(obj):
    """Set default json values for non-serializable values.

    It helps convert ``set``, ``range`` and ``np.ndarray`` data types to list.
    It also converts ``np.generic`` (including ``np.int32``, ``np.float32``,
    etc.) into plain numbers of plain python built-in types.
    """
    if isinstance(obj, (set, range)):
        return list(obj)
    elif isinstance(obj, np.ndarray):
        return obj.tolist()
    elif isinstance(obj, np.generic):
        return obj.item()
    raise TypeError(f'{type(obj)} is unsupported for json dump')


class JsonsHandler(BaseFileHandler):

    def load_from_fileobj(self, file, **kwargs):    
        rs_list = list_from_file(file, **kwargs)
        try:
            rs_list = [json.loads(s) for s in rs_list]
        except:
            pass
        return rs_list

    def dump_to_fileobj(self, objs, file, **kwargs):
        kwargs.setdefault('default', set_default)
        for obj in objs:
            json.dump(obj, file, **kwargs)
            file.write('\n')

    def dump_to_str(self, obj, **kwargs):
        kwargs.setdefault('default', set_default)
        return json.dumps(obj, **kwargs)


import json
  
from ..parse import list_from_file
from .base import BaseFileHandler


class TextsHandler(BaseFileHandler):

    def load_from_fileobj(self, file, **kwargs):    
        rs_list = list_from_file(file, **kwargs) 
        return rs_list

    def dump_to_fileobj(self, objs, file, **kwargs): 
        for obj in objs: 
            file.write(obj + '\n',**kwargs)

    def dump_to_str(self, obj, **kwargs): 
        return json.dumps(obj, **kwargs)


# -*- coding: utf-8 -*-
from loguru import logger
import time 
import functools
import sys
import os
 
# 日志耗时装饰器
def logtime(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = time.perf_counter()
        res = func(*args, **kwargs)
        end = time.perf_counter()
        logger.info('【%s】 took %.2f s' % (func.__name__, (end - start)))
        return res
    return wrapper 

'''
该日志不保存
'''
def lump_logs(guid,text):
    data_begin = f"\n-------------------- {guid} Begin --------------------\n\n"
    data_end = f"\n-------------------- {guid} End --------------------\n\n"
    logger.debug(f"{data_begin} {text} {data_end}")

##-------------------------------------------------------------------------------------------------- 
     
try:
    log_level = os.environ['LOG_LEVEL']
except KeyError:
    log_level = 3  # pylint: disable=invalid-name (C0103)
    
levels = {0: 'ERROR', 1: 'WARNING', 2: 'INFO', 3: 'DEBUG'}


def log(level=2, message="", use_color=False): 
    current_time = time.time()
    time_array = time.localtime(current_time)
    current_time = time.strftime("%Y-%m-%d %H:%M:%S", time_array)
    if log_level >= level:
        if use_color:
            print("\033[1;31;40m{} [{}]\t{}\033[0m".format(
                current_time, levels[level], message).encode("utf-8")
                  .decode("latin1"))
        else:
            print("{} [{}]\t{}".format(current_time, levels[
                level], message).encode("utf-8").decode("latin1"))
        sys.stdout.flush()


def debug(message="", use_color=False):
    log(level=3, message=message, use_color=use_color)


def info(message="", use_color=False):
    log(level=2, message=message, use_color=use_color)


def warning(message="", use_color=True):
    log(level=1, message=message, use_color=use_color)


def error(message="", use_color=True, exit=True):
    log(level=0, message=message, use_color=use_color)
    if exit:
        sys.exit(-1)


from setuptools import setup, find_packages 
import codecs
import os 

name='pipetool'

def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts), "r",encoding='utf-8').read()


def get_version(): 
    version_file = name + '/version.py'
    with open(version_file, 'r', encoding='utf-8') as f:
        exec(compile(f.read(), version_file, 'exec'))
    return locals()['__version__']
    '''
    return '0.0.1'
    '''

setup(
    name=name,
    version=get_version(),
    description="基础工具集，统一文件读写，规范流行线过程",
    author='zhys513',#作者
    packages=find_packages(),
    author_email="254851907@qq.com",
    url="https://gitee.com/zhys513/pipetool",
    python_requires='>=3.6', 
    install_requires=['addict','yapf','pyyaml','regex','packaging','loguru'],
)

